/*
 * (c) Copyright 2003 -- Frank Kirschner <kirschner@trustsec.de>
 * http://www.trustsec.de/soft/oss/
 *
 * This small program creates the home-directory of the calling user,
 * if it does not exists.
 *
 * make_home_dir needs to have a S-bit for the group, that has write
 * permission to the parent-directory of the home-directories.
 *
 * example setup:
 * All home-directorys are /home/<userid>
 * /home has to be group-writeable by the group, that make_home_dir has:
 *   $ groupadd home
 *   $ chmod g+w /home
 *   $ chgrp home /home
 *   $ gcc -Wall -o /usr/local/sbin/make_home_dir make_home_dir.c
 *   $ chgrp home /usr/local/sbin/make_home_dir
 *   $ chmod g+s /usr/local/sbin/make_home_dir
 *
 * Put the call in /etc/profile (or /etc/login for csh,tcsh)
 * A better place would be /etc/profile.d/makehomedir, if your
 * distribution supports /etc/profile.d/
 *
 * Example for bash, ksh, ash, sh:
 *     if [ ! -d $HOME ]
 *     then
 *         logger Creating new home-directory $HOME
 *         /usr/local/sbin/make_home_dir
 *         cd $HOME
 *         cp -a /etc/skel/. $HOME/.
 *         echo Home directory created
 *     fi
 */

#include <pwd.h>
#include <sys/stat.h>
#include <unistd.h>

int main(int argc, char **argv)
{
    struct passwd *pw;
    struct stat buf;
    int ret;

    pw = getpwuid(getuid());
    if ((ret = stat(pw->pw_dir,&buf)) != 0)
    {
	if ((ret = mkdir(pw->pw_dir, 0777)) == 0)
	{
	    return chown(pw->pw_dir,-1,getgid());
	}
	return ret;
    }
    return 1;
}
